
# 0 - Script information ----

#' Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
#' Script name: example7.2_with seasonality.r
#' Date created: October 2023

#' Script purpose: This r script takes the reader of the Trend analysis guidance document through each step of the analysis for example 7.2, where we carry out a trend analysis for a single geographical unit with monthly count data.
#' Please ensure your working directory is correctly set and you have a folder for images files

# Script version: v1 - Original working version
# Add here any major changes to the r script

# Please ensure your working directly is correctly set and you have a folder for image files.


# 1 - Remember to set working directory -----------------------------------

# To find out what is your current working directory
# getwd()

# If you are using an Rproject file, the working directory is already set up to the location of the Rproj file.

# Set working directory via Rstudio menu
# Session -> Set Working Directory -> "To Source File Location" (if r script already saved in the right folder)
# Session -> Set Working Directory -> "Choose Directory" (if you want to select the appropriate folder)

# Set working directory by adding code
# setwd("C:/data/...)


# 2 - Load packages and functions -------------------------------------------------------

library(dplyr)
library(lubridate)
library(tidyr)
library(EpiReport)
library(ggplot2)
library(broom)
library(lmtest)
library(ggfortify)
library(car)
library(forecast)


# Function computing manually the AIC due the issue described here:
# https://stackoverflow.com/questions/64531074/why-r-produces-incorrect-aic-and-bic
AICmanual <- function(model) {
  return(-2 * as.numeric(logLik(model)) + 2 * length(coef(model)))
}

# 3 - Import data ---------------------------------------------------------

import <-
  read.csv("./0_source_data/ECDC_surveillance_data_Salmonellosis.csv",
    stringsAsFactors = F
  )



# 4 - Preparing the data for analysis - manually -------------------------------------

# Check your data
check <- import %>%
  select(HealthTopic, Population, Indicator, Unit, RegionCode, RegionName) %>%
  unique()


#' Create, rename and select variables time, rcode, rname and count
#' Create variables for year, months and day. from the time variable. If the the month is not present it will put month 12 (end of year), if day is not present it will put day=28 (end of month). It will then create a date variable combining "year-month-day"

data <- import %>%
  # create and rename variables
  mutate(
    time = Time,
    rcode = RegionCode,
    rname = RegionName,
    count = as.numeric(NumValue) # automatically insert NA when value is missing or "-"
  ) %>%
  # select variables
  select(time, rcode, rname, count) %>%
  # create variables
  mutate(
    year = as.numeric(substr(time, 1, 4)), # select a substring of character vector (1 to 4), and convert them to numeric
    month = as.numeric(substr(time, 6, 7)),
    month = ifelse(is.na(month), "12", month), # if month is missing, assume month 12
    day = as.numeric(substr(time, 9, 10)),
    day = ifelse(is.na(day), 28, day), # if day is missing, assume day 28
    date = as.Date(paste(year, month, day, sep = "-"), "%Y-%m-%d") # create date variable, concatenate strings and convert to Date
  ) %>%
  # filter to keep data from 2010 onwards and from EU/EEA
  filter(
    year > 2009,
    rname == "EU/EEA"
  )

## Step 3: Decide on the time unit of analysis
#' Aggregate data by month
data_bymonth <- data %>%
  group_by(year, month) %>%
  summarise(count = sum(count)) %>%
  ungroup() %>% 
  # fill in any gaps in years
  complete(year = full_seq(year, 1), month = full_seq(month, 1))

sum(is.na(data_bymonth)) # check if there are any gaps in years

# For plotting using EpiReport, the x variable needs to be in the Date format
data_bymonth <- data_bymonth %>%
  # convert numeric year and month to as.Date year-month
  mutate(yearmonthdate = as.Date(paste0(year, "-", month, "-01"))) %>%
  ungroup()

## Step 4: Plot the data
# 5 - Plotting the data ---------------------------------------------------

EpiReport::plotTS(data_bymonth,
  xvar = "yearmonthdate",
  xlabel = "Year-month",
  yvar = "count",
  ylabel = "Number of Salmonellosis cases",
  log10_scale = FALSE,
  xvar_format = "%Ym%m",
  xvar_breaks = "1 year"
)

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

## Step 5: Decide on the preliminary regression type
## Step 6: Consider inclusion of seasonal terms
## Step 7: Assess the non-linearity of the trend 
## Step 8: Build the model
# 6 - Checking the non-linearity of the trend ------------------------------

#' First I code a new time variable that starts at 1 and goes to 120 (10 years x 12 months).
#' This will facilitate the next part of analysis.
#' Note that if I am interested in the intercept, this needs to be considered carefully.

data_bymonth <- data_bymonth %>%
  arrange(year, month) %>%
  # make sure data frame has been "ungrouped", otherwise it will create a sequence based on previous group
  mutate(time = seq(1:n()))





# 6.1 - Linear trend ------------------------------------------------------

#' linear regression model
#' I include the monthly seasonal term here, with February as the reference


# make variable month as a factor with February as a reference
data_bymonth <- data_bymonth %>%
  mutate(month = relevel(as.factor(month), ref = 2))

linreg <- lm(data = data_bymonth, count ~ time + month)

# Alternatively, releveling of factor can also be done within and only for the model
# linreg <- lm(data = data_bymonth, count ~ time + relevel(factor(month), ref = 2))

names(linreg) # check components of the model output

# Model outputs
summary(linreg) # using base R
tidy(linreg) # using broom package
glance(linreg) # using broom package
# augment(linreg) # using broom package

# Get the adjusted R2
summary(linreg)$adj.r.squared # using base R
glance(linreg)$adj.r.squared # using broom package


# Get the AIC - using base R
AIC(linreg)

# Get the AIC -  using broom package
glance(linreg)$AIC

# Get the AIC - Manual computation of the AIC
AICmanual(linreg)


# Plot model against the data

# transform the data to the format required by the EpiReport package
linreg_fitted <- augment(linreg) %>%
  # use the augment() function from the Broom package to get the fitted values
  select(time, count, .fitted) %>%
  # add yearmonthdate variable
  left_join(y = select(data_bymonth, time, yearmonthdate), by = "time") %>%
  rename(Data = count, Linear_model = .fitted) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Linear_model"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)

# Plot the model against the data using the EpiReport package function plotTSGrouped

plotTSGrouped(linreg_fitted,
  xvar = "yearmonthdate",
  xlabel = "Year-month",
  yvar = "Yvalue",
  ylabel = "Number of Salmonellosis cases",
  group = "Ylabel",
  log10_scale = FALSE,
  xvar_format = "%Ym%m",
  xvar_breaks = "1 year"
)

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - linear model.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)



# 6.2 - Quadratic trend ---------------------------------------------------

# create the trend variable
data_bymonth <- data_bymonth %>%
  mutate(time2 = time * time)

# run regression
quadreg <- lm(data = data_bymonth, count ~ month + time + time2)

# Model outputs
summary(quadreg) # using base R
tidy(quadreg) # using broom package
glance(quadreg) # using broom package
# augment(quadreg) # using broom package

# Get the adjusted R2
summary(quadreg)$adj.r.squared # using base R


# Get the AIC
AIC(quadreg) # using base R
AICmanual(quadreg) # using manual calculation


# Plot model against the data

# transform the data to the format required by the EpiReport package
quadreg_fitted <- augment(quadreg) %>%
  # use the augment() function from the Broom package to get the fitted values
  select(time, count, .fitted) %>%
  # add yearmonthdate variable
  left_join(y = select(data_bymonth, time, yearmonthdate), by = "time") %>%
  rename(Data = count, Quadratic_model = .fitted) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Quadratic_model"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)

# Plot the model against the data using the EpiReport package function plotTSGrouped

plotTSGrouped(quadreg_fitted,
  xvar = "yearmonthdate",
  xlabel = "Year-month",
  yvar = "Yvalue",
  ylabel = "Number of Salmonellosis cases",
  group = "Ylabel",
  log10_scale = FALSE,
  xvar_format = "%Ym%m",
  xvar_breaks = "1 year"
)

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - quadratic model.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# Compare Linear vs Quadratic trend

summary(linreg)$adj.r.squared # using base R
summary(quadreg)$adj.r.squared # using base R

AIC(linreg) # using base R
AIC(quadreg) # using base R
AICmanual(linreg)
AICmanual(quadreg)

# using ANOVA
anova(linreg)
anova(quadreg)
anova(linreg, quadreg)

# using likelihood-ratio test
lrtest(linreg, quadreg)



# 6.3 - Cubic trend ---------------------------------------------------

# create the trend variable
data_bymonth <- data_bymonth %>%
  mutate(time3 = time * time * time)

# run regression
cubreg <- lm(data = data_bymonth, count ~ month + time + time2 + time3)

# Model outputs
summary(cubreg) # using base R
tidy(cubreg) # using broom package
glance(cubreg) # using broom package
# augment(cubreg) # using broom package

# Get the adjusted R2
summary(cubreg)$adj.r.squared # using base R


# Get the AIC
AIC(cubreg) # using base R
AICmanual(cubreg)


# Plot model against the data

# transform the data to the format required by the EpiReport package
cubreg_fitted <- augment(cubreg) %>%
  # use the augment() function from the Broom package to get the fitted values
  select(time, count, .fitted) %>%
  # add yearmonthdate variable
  left_join(y = select(data_bymonth, time, yearmonthdate), by = "time") %>%
  rename(Data = count, Cubic_model = .fitted) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Cubic_model"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)

# Plot the model against the data using the EpiReport package function plotTSGrouped

plotTSGrouped(cubreg_fitted,
  xvar = "yearmonthdate",
  xlabel = "Year-month",
  yvar = "Yvalue",
  ylabel = "Number of Salmonellosis cases",
  group = "Ylabel",
  log10_scale = FALSE,
  xvar_format = "%Ym%m",
  xvar_breaks = "1 year"
)

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - cubic model.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# compare AIC
AIC(linreg)
AIC(quadreg)
AIC(cubreg)
AICmanual(linreg)
AICmanual(quadreg)
AICmanual(cubreg)

# using ANOVA
anova(quadreg, cubreg)
anova(linreg, cubreg)

# using likelihood-ratio test
lrtest(linreg, quadreg)
lrtest(quadreg, cubreg)
lrtest(linreg, cubreg)


# 6.3 - Exponential trend -------------------------------------------------

# Log transform the data
data_bymonth <- data_bymonth %>%
  mutate(logcount = log(count))


# run regression
expreg <- lm(data = data_bymonth, logcount ~ month + time)

# Model outputs
summary(expreg) # using base R
tidy(expreg) # using broom package
glance(expreg) # using broom package
augment(expreg) # using broom package

# Get the adjusted R2
summary(expreg)$adj.r.squared # using base R


# Get the AIC
AIC(expreg) # using base R
AICmanual(expreg)


# Plot model against the data

# backtransform the model values
# transform the data to the format required by the EpiReport package

expreg_fitted <- augment(expreg) %>%
  # use the augment() function from the Broom package to get the fitted values
  select(time, logcount, .fitted) %>%
  # Backtransform values from logaritmic scale
  mutate(count = exp(logcount), fitted = exp(.fitted)) %>%
  # add yearmonthdate variable
  left_join(y = select(data_bymonth, time, yearmonthdate), by = "time") %>%
  rename(Data = count, Exponential_model = fitted) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Exponential_model"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)

# Plot the model against the data using the EpiReport package function plotTSGrouped

plotTSGrouped(expreg_fitted,
  xvar = "yearmonthdate",
  xlabel = "Year-month",
  yvar = "Yvalue",
  ylabel = "Number of Salmonellosis cases",
  group = "Ylabel",
  log10_scale = FALSE,
  xvar_format = "%Ym%m",
  xvar_breaks = "1 year"
)

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - exponential model.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# compare adjusted R squared
summary(linreg)$adj.r.squared
summary(quadreg)$adj.r.squared
summary(cubreg)$adj.r.squared
summary(expreg)$adj.r.squared


## Step 9: Validate the model
# 7 - Regression diagnostics ----------------------------------------------

# 7.1 - Testing linearity assumption - Plot data points against select model -----------------------------

# To check the linearity assumption, you can plot your data against your model. The data points should be more or less symmetrically distributed around your model line.

# The code below is an adaptation of the code from function EpiReport::plotTSGrouped


model.diag.metrics <- augment(linreg) %>%
  # use the augment() function from the Broom package to get the fitted values
  select(time, count, .fitted) %>%
  left_join(y = select(data_bymonth, time, yearmonthdate), by = "time") %>%
  # add yearmonthdate variable
  rename(Data = count, Linear_model = .fitted) %>%
  pivot_longer(c("Data", "Linear_model"), names_to = "Ylabel", values_to = "Yvalue") %>%
  # transform data from wide to long format
  arrange(Ylabel, time)

# identifies the breaks for Y axis
LABELS <- pretty(seq(0,
  max(model.diag.metrics$Yvalue, na.rm = TRUE),
  by = max(model.diag.metrics$Yvalue, na.rm = TRUE) / 5
))

# uses the two main colours from EpiReport::EcdcColors(col_scale = "qual", n = 2)
fill_color <- c("#65B32E", "#7CBDC4")

ggplot(model.diag.metrics, aes(x = yearmonthdate, y = Yvalue, color = Ylabel)) +
  geom_point(data = filter(model.diag.metrics, Ylabel == "Data")) +
  geom_line(data = filter(model.diag.metrics, Ylabel == "Linear_model"), size = 1) +
  scale_colour_manual("Legend",
    values = c(fill_color),
    guide = guide_legend(override.aes = list(
      linetype = c(0, 1), # override legend definitions to keep line only
      shape = c(16, NA) # override legend definitions to keep point only
    ))
  ) +
  xlab("Year-month") +
  ylab("Number of Salmonellosis cases") +
  scale_x_date(
    date_label = "%Ym%m",
    date_breaks = "1 year",
    expand = c(0, 0)
  ) +
  scale_y_continuous(
    expand = c(0, 0),
    limits = c(0, max(LABELS, na.rm = TRUE)),
    breaks = LABELS
  ) +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - linear model scatter.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# 7.2 - Testing linearity assumption - Plot model values against residuals -----------------------------

# You can also plot the values of your model against the residuals. The residuals in the context of linear regression are what is left when you subtract your data points from your model. The points should be more or less symmetrically distributed around the horizontal

# Base R diagnostic plots
plot(linreg)
plot(linreg, 1)

# Autoplot() function within ggfortify package
autoplot(linreg)

# residualPlots() function within car package
residualPlots(linreg, ~1, fitted = TRUE, quadratic = FALSE)

# Build within ggplot
cbind(Fitted = fitted(linreg), Residuals = residuals(linreg)) %>%
  as.data.frame() %>%
  ggplot(aes(x = Fitted, y = Residuals)) +
  geom_point(size = 3, color = "#7CBDC4") +
  geom_hline(yintercept = 20, color = "#65B32E") +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - residuals over data.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)



# 7.3 - Testing Homoscedasticity assumption - Plot residuals against time -----------------------------

# The residuals are assumed to have a constant variance. If the scatter of residuals is not stable over time, then there is heteroscedasticity in your model.

# using residualPlots function
car::residualPlots(linreg, ~time, fitted = FALSE, linear = TRUE, quadratic = FALSE)

# build within ggplot

resbytime <- data_bymonth %>%
  left_join(y = augment(linreg), by = c("time", "count")) %>%
  transmute(
    Yearmonth = yearmonthdate,
    Residuals = .resid
  )

ggplot(resbytime, aes(x = Yearmonth, y = Residuals)) +
  geom_point(size = 3, color = "#7CBDC4") +
  geom_hline(yintercept = 20, color = "#65B32E") +
  xlab("Year-month") +
  ylab("Residuals") +
  scale_x_date(
    date_label = "%Ym%m",
    date_breaks = "1 year",
    expand = c(0, 0)
  ) +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )


ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - residuals over time.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# 7.4 - Testing Independence assumption - Plot model values against residuals -----------------------------

# To check the independence of your residuals you can plot your residuals over time, or create an autocorrelation graph.


# using base R
png(
  file = "./2_example7.2/images/Number of Salmonellosis cases by month - autocorrelation of residuals.png",
  width = 600,
  height = 350
)

acf(linreg$residuals, ci.type = "ma", lag.max = 40)

dev.off()

# using function ggAcf() from Forecast package
ggAcf(linreg$residuals, lag.max = 40) +
  xlab("Lag") +
  ylab("Autocorrelation of residuals") +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm"),
    plot.title = element_blank(),
  )

ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - autocorrelation of residuals.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# 7.5 - Testing Normality assumption - Plot model values against residuals -----------------------------

# To check if your residuals are normally distributed, you can create a histogram of the residuals and you can create a specific diagnostic plot called the normal probability plot (also called the quantile quantile plot).

# histogram of residuals - using base R

png(
  file = "./2_example7.2/images/Number of Salmonellosis cases by month - histogram of residuals.png",
  width = 600,
  height = 350
)

hist(linreg$residuals,
  probability = TRUE,
  breaks = "FD",
  main = "Histogram of model residuals plus normal curve",
  xlab = "Residuals"
)
curve(dnorm(x,
  mean = mean(linreg$residuals),
  sd = sd(linreg$residuals)
),
add = TRUE,
col = "green",
lwd = 2
)

dev.off()



# histogram of residuals - using ggplot

# to calculate number of bins
bw <- diff(range(linreg$residuals)) / (2 * IQR(linreg$residuals) / length(linreg$residuals)^(1 / 3))

bw <- as.integer(bw)


# to calculate breaks for equally spaced bins
# bw <- seq(min(linreg$residuals), max(linreg$residuals), length.out = 11)

ggplot() +
  geom_histogram(aes(x = linreg$residuals, y = ..density..),
     bins = bw,
     # breaks = bw,
    fill = "#65B32E", color = "black"
  ) +
  stat_function(fun = dnorm, args = list(mean = mean(linreg$residuals), sd = sd(linreg$residuals)), color = "black") +
  labs(x = "Residuals", y = "Density") +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm"),
    plot.title = element_blank(),
  )


ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - histogram of residuals.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# quantile plot for the residuals

png(file = "./2_example7.2/images/Number of Salmonellosis cases by month - qqplot of residuals.png", width = 600, height = 350)

qqnorm(linreg$residuals)
qqline(linreg$residuals)

dev.off()

# using car package
qqPlot(linreg$residuals)

# using ggplot

ggplot(linreg, aes(sample = linreg$residuals)) +
  stat_qq(color = "#7CBDC4") +
  stat_qq_line(color = "#65B32E") +
  labs(x = "Theoretical", y = "Sample") +
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm"),
    plot.title = element_blank(),
  )


ggsave(
  filename = "./2_example7.2/images/Number of Salmonellosis cases by month - qqplot of residuals.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)



# 8 - End of script -------------------------------------------------------
