
# 0 - Script information ----

#' Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
#' Script name: example4.1_HIVrates.r
#' Date created: October 2023
#' Script purpose: This r script takes the reader of the Trend analysis guidance document through each step of the analysis for example 4.1, where we carry out a trend analysis taking country into account with yearly rate data.
#' Please ensure your working directory is correctly set and you have a folder for images files

# Script version: v1 - Original working version
# Add here any major changes to the r script

# Please ensure your working directly is correctly set and you have a folder for image files.


# 1 - Remember to set working directory -----------------------------------

# To find out what is your current working directory
# getwd()

# If you are using an Rproject file, the working directory is already set up to the location of the Rproj file.

# Set working directory via Rstudio menu
# Session -> Set Working Directory -> "To Source File Location" (if r script already saved in the right folder)
# Session -> Set Working Directory -> "Choose Directory" (if you want to select the appropriate folder)

# Set working directory by adding code
# setwd("C:/data/...)


# 2 - Load packages and functions -------------------------------------------------------

library(dplyr)
library(lubridate)
library(tidyr)
library(EpiReport)
library(ggplot2)
library(broom)
library(lmtest)
library(ggfortify)
library(car)
library(forecast)
library(purrr)
library(modelr)
library(meta)

# Function computing manually the AIC due the issue described here:
# https://stackoverflow.com/questions/64531074/why-r-produces-incorrect-aic-and-bic
AICmanual <- function(model) {
  return(-2 * as.numeric(logLik(model)) + 2 * length(coef(model)))
}


# ECDC graph colours - ggplot theme

ecdctheme <- function() {
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )
}



# 3 - Import data ---------------------------------------------------------

import <-
  read.csv("./0_source_data/ECDC_surveillance_data_HIV_infection_and_AIDS.csv",
    stringsAsFactors = F
  )


# 4 - Preparing the data for analysis - manually -------------------------------------

# Check your data
check <- import %>%
  select(HealthTopic, Population, Indicator, Unit, RegionCode, RegionName) %>%
  unique()


#' Create, rename and select variables time, rcode, rname and count
#' Create variables for year, months and day. from the time variable. If the the month is not present it will put month 12 (end of year), if day is not present it will put day=28 (end of month). It will then create a date variable combining "year-month-day"

data <- import %>%
  # create and rename variables
  mutate(
    time = Time,
    rcode = RegionCode,
    rname = RegionName,
    rate = as.numeric(NumValue) # automatically insert NA when value is missing or "-"
  ) %>%
  # select variables
  select(time, rcode, rname, rate) %>%
  # create variables
  mutate(
    year = as.numeric(substr(time, 1, 4)), # select a substring of character vector (1 to 4), and convert them to numeric
    month = as.numeric(substr(time, 6, 7)),
    month = ifelse(is.na(month), "12", month), # if month is missing, assume month 12
    day = as.numeric(substr(time, 9, 10)),
    day = ifelse(is.na(day), 28, day), # if day is missing, assume day 28
    date = as.Date(paste(year, month, day, sep = "-"), "%Y-%m-%d") # create date variable, concatenate strings and convert to Date
  ) %>%
  # filter to keep data from 2007 to 2016 and exclude EU/EEA figure
  filter(
    year > 2006 & year < 2017,
    rname != "EU/EEA"
  ) %>%
  # For plotting using EpiReport, the x variable needs to be in the Date format
  # convert numeric year to as.Date year
  mutate(yeardate = as.Date(paste0(year, "-01-01"))) %>%
  select(-time)


# First I code a new time variable that starts at 1 and goes to 10.
# This will facilitate the next part of analysis.
# Note that if I am interested in the intercept, this needs to be considered carefully.

data <- data %>%
  mutate(time = year - 2006)

## Step 3: Plot the data
## Plot the raw data by country
# 5 - Plotting the data by country ---------------------------------------------------

#' We can plot the countries all together
#' Or we can plot half in one graph and half in another, by checking how many countries are available and taking the approximate half

data <- data %>%
  arrange(year, rcode) %>%
  group_by(year) %>%
  # Attribute each country a number
  mutate(country_num = seq(1:n())) %>%
  ungroup()

# calculate half of the number of countries
half <- floor(max(data$country_num) / 2)



# Plotting the countries all together

ggplot(data, aes(x = yeardate, y = rate)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~rcode) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()


ggsave(
  filename = "./3_example4.1/images/HIV notification rates all countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# Plot first half of the countries

data %>%
  filter(country_num <= half) %>%
  ggplot(aes(x = yeardate, y = rate)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~rcode) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()


ggsave(
  filename = "./3_example4.1/images/HIV notification rates first half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# Plot second half of the countries

data %>%
  filter(country_num > half) %>%
  ggplot(aes(x = yeardate, y = rate)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~rcode) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()


ggsave(
  filename = "./3_example4.1/images/HIV notification rates second half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# Step 3: Plot the data
## Plot the raw data by country and plot the trend models by country on top
# 6.1 - Plotting the data by country and the liner trend model --------------

#' First we need to create models by country.
#' We need to do this country by country using functions from purrr and modelr packages

#' nest data
#' This creates a nested data frame, i.e. a data frame where one (or more) columns is a list of data frames
by_country_linreg <- data %>%
  group_by(rcode) %>%
  nest()

# see example of nested data
by_country_linreg$data[[1]]

# model-fitting function
country_model_linreg <- function(df) {
  lm(rate ~ time, data = df)
}

# apply model fitting function to each country and add it to the nested data frame
by_country_linreg <- by_country_linreg %>%
  mutate(model = map(data, country_model_linreg)) %>%
  # add predictions via package modelr
  mutate(predictions = map2(data, model, add_predictions))

# Extract the predictions from each country model results list and add it as a column
predictions_linreg <- unnest(by_country_linreg, predictions) %>%
  rename(Data = rate, Linear_trend = pred) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Linear_trend"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)


# Plot first half of the countries
predictions_linreg %>%
  filter(country_num <= half) %>%
  ggplot(aes(x = yeardate, y = Yvalue, color = Ylabel)) +
  # create a "graph window" (facet) for each country
  facet_wrap(~rcode) +
  geom_point(data = filter(predictions_linreg, Ylabel == "Data" & country_num <= half)) +
  geom_line(data = filter(predictions_linreg, Ylabel == "Linear_trend" & country_num <= half), size = 1) +
  scale_colour_manual("Legend",
    values = c("#65B32E", "#7CBDC4"),
    guide = guide_legend(override.aes = list(
      linetype = c(0, 1), # override legend definitions to keep line only
      shape = c(16, NA) # override legend definitions to keep point only
    ))
  ) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()

ggsave(
  filename = "./3_example4.1/images/HIV notification rates and linear trend first half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)



# Plot second half of the countries
predictions_linreg %>%
  filter(country_num > half) %>%
  ggplot(aes(x = yeardate, y = Yvalue, color = Ylabel)) +
  facet_wrap(~rcode) +
  geom_point(data = filter(predictions_linreg, Ylabel == "Data" & country_num > half)) +
  geom_line(data = filter(predictions_linreg, Ylabel == "Linear_trend" & country_num > half), size = 1) +
  scale_colour_manual("Legend",
    values = c("#65B32E", "#7CBDC4"),
    guide = guide_legend(override.aes = list(
      linetype = c(0, 1), # override legend definitions to keep line only
      shape = c(16, NA) # override legend definitions to keep point only
    ))
  ) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()

ggsave(
  filename = "./3_example4.1/images/HIV notification rates and linear trend second half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# 6.2 - Plotting the data by country and the quadratic trend model --------------

#' First we need to create models by country.
#' We need to do this country by country using functions from purrr and modelr packages

# create the quadratic term
data <- data %>%
  mutate(time2 = time * time)

#' nest data
#' This creates a nested data frame, i.e. a data frame where one (or more) columns is a list of data frames
by_country_quadreg <- data %>%
  group_by(rcode) %>%
  nest()

# see example of nested data
by_country_quadreg$data[[1]]

# model-fitting function
country_model_quadreg <- function(df) {
  lm(rate ~ time + time2, data = df)
}

# apply model fitting function to each country and add it to the nested data frame
by_country_quadreg <- by_country_quadreg %>%
  mutate(model = map(data, country_model_quadreg)) %>%
  # add predictions via package modelr
  mutate(predictions = map2(data, model, add_predictions))

# Extract the predictions from each country model results list and add it as a column
predictions_quadreg <- unnest(by_country_quadreg, predictions) %>%
  rename(Data = rate, Quadratic_trend = pred) %>%
  # transform data from wide to long format
  pivot_longer(c("Data", "Quadratic_trend"), names_to = "Ylabel", values_to = "Yvalue") %>%
  arrange(Ylabel, time)


# Plot first half of the countries
predictions_quadreg %>%
  filter(country_num <= half) %>%
  ggplot(aes(x = yeardate, y = Yvalue, color = Ylabel)) +
  # create a "graph window" (facet) for each country
  facet_wrap(~rcode) +
  geom_point(data = filter(predictions_quadreg, Ylabel == "Data" & country_num <= half)) +
  geom_line(data = filter(predictions_quadreg, Ylabel == "Quadratic_trend" & country_num <= half), size = 1) +
  scale_colour_manual("Legend",
    values = c("#65B32E", "#7CBDC4"),
    guide = guide_legend(override.aes = list(
      linetype = c(0, 1), # override legend definitions to keep line only
      shape = c(16, NA) # override legend definitions to keep point only
    ))
  ) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()

ggsave(
  filename = "./3_example4.1/images/HIV notification rates and quadratic trend first half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)



# Plot second half of the countries
predictions_quadreg %>%
  filter(country_num > half) %>%
  ggplot(aes(x = yeardate, y = Yvalue, color = Ylabel)) +
  facet_wrap(~rcode) +
  geom_point(data = filter(predictions_quadreg, Ylabel == "Data" & country_num > half)) +
  geom_line(data = filter(predictions_quadreg, Ylabel == "Quadratic_trend" & country_num > half), size = 1) +
  scale_colour_manual("Legend",
    values = c("#65B32E", "#7CBDC4"),
    guide = guide_legend(override.aes = list(
      linetype = c(0, 1), # override legend definitions to keep line only
      shape = c(16, NA) # override legend definitions to keep point only
    ))
  ) +
  xlab("Year") +
  ylab("HIV notification rates per 100,000") +
  ecdctheme()

ggsave(
  filename = "./3_example4.1/images/HIV notification rates and quadratic trend second half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

## Step 4: Decide how to pool the data in the main analysis
## Step 5: Choose the time unit of analysis, consider inclusion of seasonal terms, check for non-linearity and decide on preliminary regression type
## Step 6: Build and validate the model
# 7 - Carrying out the meta-analysis --------------------------------------

#' For the meta-analysis we need to capture the linear trend estimate and its standard error for each country.
#' The nested data frame with outputs from linear models is already created "by_country_linreg"

#' Add each model key output to the data frame using the function tidy from package broom
tidy_linreg <- by_country_linreg %>%
  mutate(tidy = map(model, broom::tidy)) %>%
  unnest(tidy, .drop = TRUE)

#' Select the rcode, estimate and std.error variables as they are required for the meta-analysis models.
by_country_linreg_results <- tidy_linreg %>%
  filter(term == "time") %>%
  select(rcode, estimate, std.error)


# fixed-effects model meta-analysis

mfe <- metagen(
  data = by_country_linreg_results,
  TE = estimate,
  seTE = std.error,
  studlab = paste(rcode),
  comb.fixed = TRUE,
  comb.random = FALSE,
  prediction = TRUE,
  sm = "SMD"
)
mfe

# random-effects model meta-analysis

mre <- metagen(
  data = by_country_linreg_results,
  TE = estimate,
  seTE = std.error,
  studlab = paste(rcode),
  comb.fixed = FALSE,
  comb.random = TRUE,
  method.tau = "DL",
  hakn = FALSE,
  prediction = TRUE,
  sm = "SMD"
)

mre

# Generate the forest plot

forest(mre)

# Export the forest plot

png(
  file = "./3_example4.1/images/HIV notification rates and linear trend - random effect meta-analysis.png",
  width = 823,
  height = 734
)

forest(mre)

dev.off()



# 8 - End of script -------------------------------------------------------
