
# 0 - Script information ----

#' Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
#' Script name: example3.1_SalmoMissingData.r
#' Date created: October 2023
#' Script purpose: This r script takes the reader of the Trend analysis guidance document through each step of the analysis for example 3.1, where we carry out a trend analysis for with yearly count data where there is a large amount of missing data for two countries.
#' #' Please ensure your working directory is correctly set and you have a folder for images files

# Script version: v1 - Original working version
# Add here any major changes to the r script

# Please ensure your working directly is correctly set and you have a folder for image files.


# 1 - Remember to set working directory -----------------------------------

# To find out what is your current working directory
# getwd()

# If you are using an Rproject file, the working directory is already set up to the location of the Rproj file.

# Set working directory via Rstudio menu
# Session -> Set Working Directory -> "To Source File Location" (if r script already saved in the right folder)
# Session -> Set Working Directory -> "Choose Directory" (if you want to select the appropriate folder)

# Set working directory by adding code
# setwd("C:/data/...)


# 2 - Load packages and functions -------------------------------------------------------

library(dplyr)
library(lubridate)
library(tidyr)
library(EpiReport)
library(ggplot2)
library(broom)
library(lmtest)
library(ggfortify)
library(car)
library(forecast)
library(purrr)
library(modelr)
library(meta)
library(janitor)
library(naniar)


# ECDC graph colours - ggplot theme

ecdctheme <- function() {
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )
}



# 3 - Import data ---------------------------------------------------------

import <-
  read.csv("./0_source_data/ECDC_surveillance_data_Salmonellosis_2countriesmissing.csv",
    stringsAsFactors = F
  )


# 4 - Preparing the data for analysis - manually -------------------------------------

# Check your data
check <- import %>%
  select(healthtopic, population, indicator, unit, country) %>%
  unique()

# First I code a new time variable that starts at 1 and goes to 10.
# This will facilitate the next part of analysis.
# Note that if I am interested in the intercept, this needs to be considered carefully.

data_byyear <- import %>%
  # For plotting using EpiReport, the x variable needs to be in the Date format
  mutate(yeardate = as.Date(paste0(year, "-01-01"))) %>%
  mutate(time = year - 2006)


## Step 3: Describe the missing data
# 5 - Describe the missing data -------------------------------------------

# Creating a dummy variable for "being missing", which will help with the descriptive analysis

data_byyear <- data_byyear %>%
  mutate(missing = case_when(
    is.na(count) ~ 1,
    TRUE ~ 0
  ))

# Overall proportion of missing data

# visualise number of missing by variable
miss_var_summary(data_byyear)

# Now focusing on the variable count, calculate number of missing and number complete
n_miss(data_byyear$count)
n_complete(data_byyear$count)

# calculate proporiton of missing in variable count
prop_miss(data_byyear$count)

# produces table of missing count
data_byyear %>%
  select(count) %>%
  miss_case_table()

# Table of missing values over time and country

table_missing <- data_byyear %>%
  group_by(year, country) %>%
  select(count) %>%
  miss_var_summary() %>%
  select(-variable, -pct_miss) %>%
  pivot_wider(names_from = year, values_from = n_miss) %>%
  ungroup()


# visualise variables with missing
data_byyear %>%
  gg_miss_fct(fct = country)

# visualise missing points by year and country
# NB: The approach of naniar package is to visualise missing data 
# by replacing NA values with values 10% lower than the minimum value in that variable
ggplot(data = data_byyear, aes(x = year, y = count)) +
  geom_point(na.rm = TRUE) +
  geom_miss_point() +
  scale_color_manual(values = c("darkorange", "cyan4")) +
  theme_classic() +
  facet_wrap(~country, scales = "free")

ggsave(
  filename = "./5_example3.1/images/Number of Salmonelosis cases by country and year - by missing status.png",
  plot = last_plot(),
  device = "png",
  width = 9,
  height = 5,
  units = c("in")
)


# Table of missing values by country

table_complete_country <- data_byyear %>%
  group_by(country) %>%
  select(count) %>%
  miss_var_summary() %>%
  select(-variable, -n_miss) %>%
  ungroup() %>%
  mutate(pct_complete = 100 - pct_miss)

# Table of missing values over time

table_complete_year <- data_byyear %>%
  group_by(year) %>%
  select(count) %>%
  miss_var_summary() %>%
  select(-variable, -n_miss) %>%
  ungroup() %>%
  mutate(pct_complete = 100 - pct_miss)


# 6 - Plotting the data by country ---------------------------------------------------

#' We can plot the countries all together
#' Or we can plot half in one graph and half in another, by checking how many countries are available and taking the approximate half

data_byyear <- data_byyear %>%
  arrange(year, country) %>%
  group_by(year) %>%
  # Attribute each country a number
  mutate(country_num = seq_along(1:n())) %>%
  ungroup()

# calculate half of the number of countries
half <- floor(max(data_byyear$country_num) / 2)

# Plotting the countries all together

ggplot(data_byyear, aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()

ggsave(
  filename = "./5_example3.1/images/Salmonella confirmed cases all countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# Plot first half of the countries

data_byyear %>%
  filter(country_num <= half) %>%
  ggplot(aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()

ggsave(
  filename = "./5_example3.1/images/Salmonella confirmed cases first half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)


# Plot second half of the countries

data_byyear %>%
  filter(country_num > half) %>%
  ggplot(aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()

ggsave(
  filename = "./5_example3.1/images/Salmonella confirmed cases second half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# Or we can plot specifically those countries with missing data
countries_withmiss <- data_byyear %>%
  filter(is.na(count)) %>%
  distinct(country) %>%
  pull() 

ggplot(data_byyear %>% filter(country %in% countries_withmiss), 
       aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()

ggsave(
  filename = "./5_example3.1/images/Salmonella confirmed cases countries with missing values.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)




## Step 4: Choose the approach for handling the missing data
## Step 5: Carry out a complete case analysis
## Step 8: Estimate the trend
# 7 - Calculating the trend -----------------------------------------------


# We decide to do a complete case analysis and include only countries that have no missing data

countries_withmiss <- data_byyear %>%
  filter(is.na(count)) %>%
  distinct(country) %>%
  pull() 

data_byyear_nomiss <- data_byyear %>%
  filter(!(country %in% countries_withmiss))


#' Then we need to create models by country.
#' We need to do this country by country using functions from purrr and modelr packages

#' nest data
#' This creates a nested data frame, i.e. a data frame where one (or more) columns is a list of data frames
by_country_linreg <- data_byyear_nomiss %>%
  group_by(country) %>%
  nest()

# see example of nested data
by_country_linreg$data[[1]]

# model-fitting function
country_model_linreg <- function(df) {
  lm(count ~ time, data = df)
}

# apply model fitting function to each country and add it to the nested data frame
by_country_linreg <- by_country_linreg %>%
  mutate(model = map(data, country_model_linreg))




# 8 - Carrying out the meta-analysis --------------------------------------

#' For the meta-analysis we need to capture the linear trend estimate and its standard error for each country.
#' The nested data frame with outputs from linear models is already created "by_country_linreg"

#' Add each model key output to the data frame using the function tidy from package broom
tidy_linreg <- by_country_linreg %>%
  mutate(tidy = map(model, broom::tidy)) %>%
  unnest(tidy)

#' Select the country, estimate and std.error variables as they are required for the meta-analysis models.
by_country_linreg_results <- tidy_linreg %>%
  filter(term == "time") %>%
  select(country, estimate, std.error)

# random-effects model meta-analysis

mre <- metagen(
  data = by_country_linreg_results,
  TE = estimate,
  seTE = std.error,
  studlab = paste(country),
  comb.fixed = FALSE,
  comb.random = TRUE,
  method.tau = "DL",
  hakn = FALSE,
  prediction = TRUE,
  sm = "SMD"
)

mre

# Generate the forest plot

forest(mre)

# Export the forest plot

png(
  file = "./5_example3.1/images/Salmonella confirmed cases and linear trend - random effect meta-analysis.png",
  width = 823,
  height = 734
)

forest(mre)

dev.off()


# 9 - End of script -------------------------------------------------------
