
# 0 - Script information ----

#' Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
#' Script name: example3.2_multiple imputation.r
#' Date created: October 2023
#' Script purpose: This r script takes the reader of the Trend analysis guidance document through each step of the analysis for example 3.2, where we carry out a trend analysis for with yearly count data where there is a small amount of missing data for some countries.
#' #' Please ensure your working directory is correctly set and you have a folder for images files

# Script version: v1 - Original working version
# Add here any major changes to the r script

# Please ensure your working directly is correctly set and you have a folder for image files.


# 1 - Remember to set working directory -----------------------------------

# To find out what is your current working directory
# getwd()

# If you are using an Rproject file, the working directory is already set up to the location of the Rproj file.

# Set working directory via Rstudio menu
# Session -> Set Working Directory -> "To Source File Location" (if r script already saved in the right folder)
# Session -> Set Working Directory -> "Choose Directory" (if you want to select the appropriate folder)

# Set working directory by adding code
# setwd("C:/data/...)


# 2 - Load packages and functions -------------------------------------------------------

library(dplyr)
library(lubridate)
library(tidyr)
library(EpiReport)
library(ggplot2)
library(broom)
library(lmtest)
library(ggfortify)
library(car)
library(forecast)
library(purrr)
library(modelr)
library(meta)
library(janitor)
library(naniar)
library(mice)
library(stringr)



# ECDC graph colours - ggplot theme

ecdctheme <- function() {
  theme(
    axis.text = element_text(size = 8),
    axis.title = element_text(size = 9),
    axis.line = element_line(colour = EcdcColors(col_scale = "grey", grey_shade = "mediumdark")),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.background = element_blank(),
    legend.position = "right",
    legend.title = element_blank(),
    legend.text = element_text(size = 8),
    legend.key = element_blank(),
    legend.key.width = unit(0.8, "cm")
  )
}



# 3 - Import data ---------------------------------------------------------


import <-
  read.csv("./0_source_data/ECDC_surveillance_data_Salmonellosis_5missing.csv",
           stringsAsFactors = F
  )



# 4 - Data preparation ----------------------------------------------------

data <- import %>%
  # Creating new time variable starting at 1 to facilitate the next part of analysis
  mutate(time = year - 2006) %>%
  # For plotting using EpiReport, the x variable needs to be in the Date format
  # convert numeric year to as.Date year
  mutate(yeardate = as.Date(paste0(year, "-01-01"))) 


## Step 3: Describe the missing data
# 5 - Table of missing values over time and country -----------------------

missing_table <- data %>%
  # Creating a dummy variable for "being missing", which will help with the descriptive analysis
  mutate(missing = ifelse( is.na(count), 1, 0)) %>%
  # Keeping only relevant variables
  select(year, country, missing) %>%
  # Table of missing values over time and country
  pivot_wider(names_from = year, values_from = missing)



# 6 - Completeness for individual years -------------------------------------

complete_year <- data %>%
  # Creating a dummy variable for "being missing", which will help with the descriptive analysis
  mutate(complete = ifelse( is.na(count), 0, 1)) %>%
  # Completeness for individual years
  group_by(year) %>%
  summarise(num = sum(complete, na.rm = TRUE),
            den = n(),
            pc = round(num / den * 100, 0))


# 7 - Completeness for individual country ----------------------------------

complete_country <- data %>%
  # Creating a dummy variable for "being missing", which will help with the descriptive analysis
  mutate(complete = ifelse( is.na(count), 0, 1)) %>%
  # Completeness for individual years
  group_by(country) %>%
  summarise(num = sum(complete, na.rm = TRUE),
            den = n(),
            pc = round(num / den * 100, 0))


# 8 - Plotting completeness -----------------------------------------------

ggplot(data, aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()

ggsave(
  filename = "./6_example3.2/images/Salmonella confirmed cases all countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# Or we can plot half of the countries on one graph, splitting the countries in half

data <- data %>%
  mutate(country_num = as.integer(as.factor(country))) 

# Graphing the first half  
ggplot(data %>% filter(country_num < max(country_num/2)), aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()
ggsave(
  filename = "./6_example3.2/images/Salmonella confirmed cases first half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)
# Graphing the second half
ggplot(data %>% filter(country_num >= max(country_num/2)), aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()
ggsave(
  filename = "./6_example3.2/images/Salmonella confirmed cases second half of countries.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# Or we can plot specifically those countries with missing data
countries_withmiss <- data %>%
  filter(is.na(count)) %>%
  distinct(country) %>%
  pull() 
ggplot(data %>% filter(country %in% countries_withmiss), aes(x = yeardate, y = count)) +
  geom_line(color = "#65B32E") +
  facet_wrap(~country) +
  xlab("Year") +
  ylab("Salmonella confirmed cases") +
  ecdctheme()
ggsave(
  filename = "./6_example3.2/images/Salmonella confirmed cases countries with missing values.png",
  plot = last_plot(),
  device = "png",
  width = 7,
  height = 4,
  units = c("in")
)

# Removing 
rm(complete_country, complete_year, missing_table)

## Step 4: Choose the approach for handling the missing data
## Step 6: Carry out the multiple imputation
# 9 - Multiple imputation - v1 -------------------------------------------------
# Using "predictive mean matching" (pmm) method 

df <- data %>%
  select(year, count, country) %>%
  # Removing space character for data manipulation
  mutate(country_code = str_remove(country, " "))

# Setting number of imputations
M <- 5

# Impute missing values with "predictive mean matching" method 
data_imp <- mice(df, m = M, method = "pmm") 

# Completing the data with the first imputation computed
data_imp1 <- complete(data_imp)  

# OR
# Completing the data with all 5 imputation datasets
data_imp1 <- data_imp %>% 
  complete(action = "long") %>%
  select(year, country_code, count, .imp) %>%
  pivot_wider(values_from = count, names_from = .imp) 
# Adding mean and sd of the 5 imputations
data_imp1 <- data_imp1 %>%
  mutate(mean = apply(data_imp1[,(1:M)+2], 1, mean),
         sd = apply(data_imp1[,(1:M)+2], 1, sd))

# Saving imputed dataset
saveRDS(data_imp1, file = "./6_example3.2/ECDC_surveillance_data_Salmonellosis_5missing_recoded_MICE1.rds")



# 9 - Multiple imputation - v2 -------------------------------------------------
# Using pmm method and specific predictive matrix

# Setting number of imputations
M <- 5

# Preparing the data
df <- data %>%
  # Removing space character for data manipulation
  mutate(country_code = str_remove(country, " ")) %>%
  select(year, count, country_code) %>%
  # Moving to wide format for imputation
  pivot_wider(names_from = country_code, values_from = count)

# Preparing imputation matrix  
RC <- names(df)[-1]
LP <- list()
for(n in RC) LP[[n]] <- n 
predMatrix <- cbind(rep(0, length(RC)), 1 - diag(length(RC)))

# Imputation
Id <- mice(df, 
           blocks = LP, 
           predictorMatrix = predMatrix, 
           m = M, 
           printFlag = F)

# Building imputed dataset with the mean/sd of the 5 imputations
data_imp2 <- spread(gather(complete(Id, "long"), "country_code", "count", -c(1:3)), .imp, count)[,-1] 
data_imp2 <- data_imp2 %>%
  mutate(mean = apply(data_imp2[,(1:M)+2], 1, mean),
         sd = apply(data_imp2[,(1:M)+2], 1, sd))

# # Mean of the 5 imputations for final dataset
# data_imp2 <- data_imp2 %>%
#   select(year, country_code, me) %>%
#   rename(count_imp = me)

# Saving imputed dataset
saveRDS(data_imp2, file = "./6_example3.2/ECDC_surveillance_data_Salmonellosis_5missing_recoded_MICE2.rds")


## Step 7: Carry out diagnostics after the imputation

# 10 - Plotting the imputed values ----------------------------------------

# Selecting the method of interest
data_imp <- data_imp2

# Preparing long format
data_imp_long <- data_imp %>%
  pivot_longer(cols = (1:M)+2 , names_to = "nb_imp") %>%
  distinct(year, country_code, value, .keep_all = TRUE)

# Identifying countries with imputed values
country_imp <- data_imp_long %>%
  filter(nb_imp > 1) %>%
  distinct(country_code) %>%
  arrange(country_code) %>%
  pull(country_code)

# Plotting
ggplot(data_imp_long %>% filter(country_code %in% country_imp), 
       aes(x = year, y = value, color = nb_imp)) +
  geom_point() +
  facet_wrap(~country_code) +
  scale_color_manual(values = EpiReport::EcdcColors("qual", n = 5)) +
  xlab("Year") +
  ylab("Salmonella cases") +
  coord_cartesian(ylim = c(0, NA))+
  ecdctheme()


## Step 8: Estimate the trend
# 11 - Computing trend -----------------------------------------------------

#' First we need to create models by country.
#' We need to do this country by country using functions from purrr 

by_country_linreg <- data_imp_long %>%
  # Keeping only one value for each imputation 
  # (in the model later on, we will select mean variable)
  filter(nb_imp == 1) %>%
  # New time variable that starts at 1 and goes to 10.
  mutate(time = year - 2006) %>%
  group_by(country_code) %>%
  # This creates a nested data frame, i.e. a data frame where one (or more) columns is a list of data frames
  nest()

# see example of nested data
by_country_linreg$data[[1]]

# model-fitting function
country_model_linreg <- function(df) {
  lm(mean ~ time, data = df)
}

# apply model fitting function to each country and add it to the nested data frame
by_country_linreg <- by_country_linreg %>%
  mutate(model = map(data, country_model_linreg))


# 12 - Carrying out the meta-analysis --------------------------------------

#' For the meta-analysis we need to capture the linear trend estimate and its standard error for each country.
#' The nested data frame with outputs from linear models is already created "by_country_linreg"

#' Add each model key output to the data frame using the function tidy from package broom
tidy_linreg <- by_country_linreg %>%
  mutate(tidy = map(model, broom::tidy)) %>%
  unnest(tidy)

#' Select the country, estimate and std.error variables as they are required for the meta-analysis models.
by_country_linreg_results <- tidy_linreg %>%
  filter(term == "time") %>%
  select(country_code, estimate, std.error)

# random-effects model meta-analysis

mre <- metagen(
  data = by_country_linreg_results,
  TE = estimate,
  seTE = std.error,
  studlab = paste(country_code),
  comb.fixed = FALSE,
  comb.random = TRUE,
  method.tau = "DL",
  hakn = FALSE,
  prediction = TRUE,
  sm = "SMD"
)

mre

# Generate the forest plot

forest(mre)

# Export the forest plot

png(
  file = "./6_example3.2/images/Salmonella confirmed cases and linear trend - random effect meta-analysis.png",
  width = 823,
  height = 734
)

forest(mre)

dev.off()


## Step 5: Complete case analysis
# 13 - Comparing with complete case analysis -------------------------------

# We include only countries that have no missing data
countries_withmiss <- data %>%
  filter(is.na(count)) %>%
  distinct(country) %>%
  mutate(country_code = str_remove(country, " ")) %>%
  pull(country_code) 

data_imp_long_nomiss <- data_imp_long %>%
  filter(!(country_code %in% countries_withmiss))

# Computing trend
by_country_linreg <- data_imp_long_nomiss %>%
  # Keeping only one value for each imputation 
  # (in the model later on, we will select mean variable)
  filter(nb_imp == 1) %>%
  # New time variable that starts at 1 and goes to 10.
  mutate(time = year - 2006) %>%
  group_by(country_code) %>%
  # This creates a nested data frame, i.e. a data frame where one (or more) columns is a list of data frames
  nest()

# see example of nested data
by_country_linreg$data[[1]]

# model-fitting function
country_model_linreg <- function(df) {
  lm(mean ~ time, data = df)
}

# apply model fitting function to each country and add it to the nested data frame
by_country_linreg <- by_country_linreg %>%
  mutate(model = map(data, country_model_linreg))

# Add each model key output to the data frame using the function tidy from package broom
tidy_linreg <- by_country_linreg %>%
  mutate(tidy = map(model, broom::tidy)) %>%
  unnest(tidy)

# Select the country, estimate and std.error variables as they are required for the meta-analysis models.
by_country_linreg_results <- tidy_linreg %>%
  filter(term == "time") %>%
  select(country_code, estimate, std.error)

# random-effects model meta-analysis
mre <- metagen(
  data = by_country_linreg_results,
  TE = estimate,
  seTE = std.error,
  studlab = paste(country_code),
  comb.fixed = FALSE,
  comb.random = TRUE,
  method.tau = "DL",
  hakn = FALSE,
  prediction = TRUE,
  sm = "SMD"
)

mre

# Generate the forest plot
forest(mre)

# Export the forest plot
png(
  file = "./6_example3.2/images/Salmonella confirmed cases and linear trend - random effect meta-analysis - excluding missing.png",
  width = 823,
  height = 734
)

forest(mre)

dev.off()


# 14 - End of script -------------------------------------------------------









