**********************************************************************
* Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
* Example 7.2 Salmo with seasonality.do
* Date created: October 2023
* Purpose:
/*
This do-file takes the reader of the Trend analysis guidance document through
each step of the analysis for example 7.2, where we carry out a trend analysis
for a single geographical unit with monthly count data.
Please ensure your working directory is correctly set and you have a folder for log 
files and image files.
*/
*
**********************************************************************

// v1
// - Original working version
* Add here any major changes to the do-file

version 15.1


********************
* Opening log file *
********************

capture log close
log using "log/Example 7.2 Salmo with seasonality", text replace


*********************
* Opening data file *
*********************

import delimited "ECDC_surveillance_data_Salmonellosis.csv", clear


***********************************
* Preparing the data for analysis *
***********************************

// Extracting the year from the month string variable
* First create a monthly date
gen mdate = monthly(time, "YM")
format mdate %tm

** Keeping data from 2010  onwards
gen date = dofm(mdate)
format date %td
gen year=year(date)
keep if year(date)>2009

// Preparing the count variable
gen count = numvalue
replace count = "" if count=="-"
destring count, replace

// Step 3: Decide on the time unit of analysis
// Creating a month variable
gen month=month(date)

// We are only interested in EU/EEA totals
keep if regionname=="EU/EEA"

//  Aggregating to year dates
collapse (sum) count, by(mdate year month)
tsset mdate
tsfill	// checking if there are missing years

// Step 4: Plot the data
*********************
* Plotting the data *
*********************

line count mdate, xtitle("Year-month") ytitle("Number of Salmonellosis cases") ylabel(0(5000)20000) 

** Saving the graph
graph export "images/Number of Salmonellosis cases by month.png", replace

// Step 5: Decide on the preliminary regression type
// Step 6: Consider inclusion of seasonal terms
// Step 7: Assess the non-linearity of the trend 
******************************************
* Checking the nonlinearity of the trend *
******************************************

// First I code a new time variable that starts at 1 and goes to 120 (10 years x 12 months). 
// This will facilitate the next part of analysis.
// Note that if I am interested in the intercept, this needs to be considered carefully.
sort mdate
gen time = _n

// First linear trend: look for adjusted R2, AIC
* I include the monthly seasonal term here, with February as the reference
regress count ib2.month time
* The adjusted R2 is:
di e(r2_a)
* The AIC is:
estat ic
matrix A = r(S)
di A[1,5]

// Plot model against the data
predict model1, xb
sort mdate
line count model1 mdate, xtitle("Year-month") ytitle("Number of Salmonellosis cases") ylabel(0(5000)20000) legend(order(1 "Data" 2 "Linear model")) 
** Saving the graph
graph export "images/Number of Salmonellosis cases by month - linear model.png", replace
// Quadratic trend
* Creating trend variable
gen time2=time*time

* Regression:
regress count ib2.month time time2
* Alternative code: regress count time c.time#c.time
* The adjusted R2 is:
di e(r2_a)
* The AIC is:
estat ic
matrix A = r(S)
di A[1,5]

// Plot model against the data
predict model2, xb

line count model2 mdate, xtitle("Year-month") ytitle("Number of Salmonellosis cases") ylabel(0(5000)20000) legend(order(1 "Data" 2 "Quadratic model")) 
graph export "images/Number of Salmonellosis cases by month - quadratic model.png", replace

// LR test
regress count ib2.month time
estimates store A

regress count ib2.month time time2
estimates store B
lrtest A

// Cubic trend
* Creating trend variable
gen time3=time*time*time

* Regression:
regress count ib2.month time time2 time3
* Alternative code: regress count time c.time#c.time c.time#c.time#c.time
* The adjusted R2 is:
di e(r2_a)
* The AIC is:
estat ic
matrix A = r(S)
di A[1,5]

// Plot model against the data
predict model3, xb

line count model3 mdate, xtitle("Year-month") ytitle("Number of Salmonellosis cases") ylabel(0(5000)20000) legend(order(1 "Data" 2 "Cubic model")) 
graph export "images/Number of Salmonellosis cases by month - cubic model.png", replace

// LR test
regress count ib2.month time time2 time3
estimates store C

lrtest B C
lrtest A C

// Exponential trend
* Log transform data
gen logcount=log(count)
regress logcount ib2.month time
* The adjusted R2 is:
di e(r2_a)
* The AIC is:
estat ic
matrix A = r(S)
di A[1,5]
// Plot model against the data
predict model4log, xb

* I backtransform the model values
gen model4=exp(model4log)

line count model4 mdate, xtitle("Year-month") ytitle("Number of Salmonellosis cases") ylabel(0(5000)20000)legend(order(1 "Data" 2 "Exponential model")) 
graph export "images/Number of Salmonellosis cases by month - exponential model.png", replace

// Step 8: Build the model
***************
* Final model *
***************

regress count ib2.month time

// Step 9: Validate the model
*************************
* Regresion diagnostics *
*************************

// Linearity assumption
twoway (scatter count mdate) (line model1 mdate), xtitle("Year-month") ytitle("Number of Salmonellosis cases") /// 
ylabel(0(5000)20000) legend(order(1 "Data" 2 "Linear model"))
graph export "images/Number of Salmonellosis cases by month - linear model scatter.png", replace

rvfplot, yline(0)
graph export "images/Number of Salmonellosis cases  by month - residuals over data.png", replace

// Homoscedasticity assumption
regress count ib2.month time
predict res, res
scatter  res mdate, xtitle(Year) ytitle("Residuals from linear model")  yline(0)
graph export "images/Salmonellosis by month - residuals over time.png", replace
 
// Independence assumption
ac res, ytitle("Autocorrelation of residuals")
graph export "images/Salmonellosis by month - autocorrelation of residuals.png", replace

// Normality assumption
histogram res,  freq normal
graph export "images/Salmonellosis by month - histogramme of residuals.png", replace
qnorm res
graph export "images/Salmonellosis by month - qqplot of residuals.png", replace

capture log close

