**********************************************************************
* ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
* Example 4.2 Hepatitis B rates.do
* Date created: October 2023
* Purpose:
/*
This do-file takes the reader of the Trend analysis guidance document through
each step of the analysis for example 4.2, where we carry out a trend analysis
taking country into account with yearly rate data.
Please ensure your working directory is correctly set and you have a folder for log 
files and image files.
*/
*
**********************************************************************

// v1
// - Original working version
* Add here any major changes to the do-file

version 15.1


********************
* Opening log file *
********************

capture log close
log using "log/Example 4.2 Hepatitis B rates", text replace


*********************
* Opening data file *
*********************

import delimited "ECDC_surveillance_data_Hepatitis_B.csv", clear


***********************************
* Preparing the data for analysis *
***********************************

// Dropping countries where we do not have complete data
drop if regioncode=="BE"
drop if regioncode=="HR"

// Changing rate from string to numeric and creating it in a new variable
destring numvalue, gen(rate)

// Keeping data from 2008 onwards for 10 years
keep if time>2007 & time<2018

// We are only interested in individual data
drop if regioncode=="EU_EEA31"

// Preparing the rate and year variable (this is not technically necessary, but harmonises to 
// previous examples)
rename time year

// First I code a new time variable that starts at 1 and goes to 10. 
// This will facilitate the next part of analysis.
// Note that if I am interested in the intercept, this needs to be considered carefully.
gen time = year-2007

// Step 3: Plot the data
// Plot the raw data by country
********************************
* Plotting the data by country *
********************************

// We can plot the countries altogether:
line rate year , xtitle("Year", size(small)) ytitle("Hepatitis B notification rates", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(allcountries, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) lcolor("101 179 46") graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white))
graph export "images/Hepatitis B notification rates all countries.png", replace

// Or we can plot half of the countries on one graph, splitting the countries in half
// by checking how many countries are available and taking the approximate half
encode regioncode, gen(country_num)
sum country_num
local HALF = floor(r(max)/2)

// Graphing the first half
line rate year if country_num<=`HALF', xtitle("Year", size(small)) ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(firsthalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white))
** Saving the graph
graph export "images/Hepatitis B notification rates first half of countries.png", replace

// Graphing the second half
line rate year if country_num>`HALF', xtitle("Year", size(small)) ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(secondhalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white))
** Saving the graph
graph export "images/Hepatitis B notification rates second half of countries.png", replace

// Step 3: Plot the data
// Plot the raw data by country and plot the trend models by country on top
***********************************************************
* Plotting the data by country and the linear trend model *
***********************************************************

// First we need to create models by country. First we use linear regression for linear trends
* We need to do this country by country, so we create a local macro with all country names:
levelsof regioncode, local(COUNTRIES)
* we store the values of trend in the new trend variable
gen trend =.
* Then we loop around each country:
foreach COUNTRY in `COUNTRIES' {
    di "`COUNTRY'"
regress rate time if regioncode=="`COUNTRY'"
predict xb if regioncode=="`COUNTRY'"
replace trend = xb if trend==. & xb!=.
drop xb
}

// Once again we probably want to plot half the countries on one graph:
sum country_num
local HALF = floor(r(max)/2)

// Graphing first half
twoway  (scatter rate year, msize(vsmall) mcolor("101 179 46") lcolor("101 179 46")) (line trend year, mcolor("101 179 46") lcolor("101 179 46")) if country_num<=`HALF', ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(firsthalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) legend(lab(1 "Data") lab(2 "Linear trend"))
** Saving the graph
graph export "images/Hepatitis B notification rates and linear trend first half of countries.png", replace

// Graphing second half
twoway  (scatter rate year, msize(vsmall) mcolor("101 179 46") lcolor("101 179 46")) (line trend year, mcolor("101 179 46") lcolor("101 179 46")) if country_num>`HALF', ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(secondhalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) legend(lab(1 "Data") lab(2 "Linear trend"))
** Saving the graph
graph export "images/Hepatitis B notification rates and linear trend second half of countries.png", replace



***********************************************************
* Plotting the data by country and the quadratic trend model *
***********************************************************

// First we need to create models by country. First we use linear regression for quadratic trends
* We create the quadratic term
gen time2 = time*time
* We need to do this country by country, so we create a local macro with all country names:
levelsof regioncode, local(COUNTRIES)
* we store the values of trend in the new trend variable
capture drop trend
gen trend =.
* Then we loop around each country:
foreach COUNTRY in `COUNTRIES' {
    di "`COUNTRY'"
regress rate time time2 if regioncode=="`COUNTRY'"
predict xb if regioncode=="`COUNTRY'"
replace trend = xb if trend==. & xb!=.
drop xb
}

// Once again we probably want to plot half the countries on one graph:
sum country_num
local HALF = floor(r(max)/2)

// Graphing first half
twoway  (scatter rate year, msize(vsmall) mcolor("101 179 46") lcolor("101 179 46")) (line trend year, mcolor("101 179 46") lcolor("101 179 46")) if country_num<=`HALF', ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(firsthalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) legend(lab(1 "Data") lab(2 "Quadratic trend"))
** Saving the graph
** Saving the graph
graph export "images/Hepatitis B notification rates and quadratic trend first half of countries.png", replace

// Graphing second half
twoway  (scatter rate year, msize(vsmall) mcolor("101 179 46") lcolor("101 179 46")) (line trend year, mcolor("101 179 46") lcolor("101 179 46")) if country_num>`HALF', ytitle("Hepatitis B notification rates per 100,000", size(small)) by(regioncode, graphregion(fcolor(white)) note("")) name(secondhalf, replace)  xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) graphregion(margin(medium) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) legend(lab(1 "Data") lab(2 "Quadratic trend"))
** Saving the graph
graph export "images/Hepatitis B notification rates and quadratic trend second half of countries.png", replace


// Step 4: Decide how to pool the data in the main analysis
// Step 5: Choose the time unit of analysis, consider inclusion of seasonal terms, check for non-linearity and decide on preliminary regression type
// Step 6: Build and validate the model
**********************************
* Carrying out the meta-analysis *
**********************************

// For the meta-analysis we need to capture the trend estimate and its standard for each country. There are different ways to do this. Here we choose to use a matrix.
// For the puproses of exercise 4.2 we do this mainly to obtain the statistical tests (Q-statistic p-value and I2 index), rather than to obtain the pooled estimate.

* Capturing the number of countries
tab regioncode
local SUM = r(r)

* Creating matrix with the same number of rows as there are countries
* And with two columns
matrix A=J(`SUM',2,0)

* Giving rownames
levelsof regioncode, local(COUNTRIES)
matrix rownames A`mat'= `COUNTRIES'

* Giving column names
matrix colnames A= "Coeff" "SE" 

* Capturing coefficient and Standard Error
local j = 1
foreach COUNTRY in `COUNTRIES' {
regress rate time if regioncode=="`COUNTRY'"
matrix A[`j',1]=_b[time]
matrix A[`j',2]=_se[time]
local j = `j' + 1
}

* checking it all looks OK
matrix list A

* Changing matrix to data
svmat A, names(col) 

* Ensuring that we have the correct country associated with the coefficient and SE:
local rownames : rowfullnames A
local c : word count `rownames'
gen study = ""
forvalues i = 1/`c' {
    replace study = "`:word `i' of `rownames''" in `i'
}

order study, before(Coeff)

keep study Coeff SE

// Stata 16 commands for meta-analysis
meta set Coeff SE, studylabel(study) 
meta summarize, random(dlaird) sort(_meta_id, ascending)
meta forestplot, random(dlaird) sort(_meta_id, ascending)


capture log close

