**********************************************************************
* ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
* Example 3.1 Salmo missing data.do
* Date created: October 2023
* Purpose:
/*
This do-file takes the reader of the Trend analysis guidance document through
each step of the analysis for example 3.1, where we carry out a trend analysis
for with yearly count data where there is a large amount of missing data for two countries.
Please ensure your working directory is correctly set and you have a folder for log 
files and image files.
*/
*
**********************************************************************

// v1
// - Original working version
* Add here any major changes to the do-file

version 16.1


********************
* Opening log file *
********************

capture log close
log using "log/Example 3.1 Salmo missing data", text replace


*********************
* Opening data file *
*********************

// Here we open an already prepared dataset, where we have coded values for 7 years to missing 
// We also mask the country name here, as in fact, no country has actually reported missing data
use "ECDC_surveillance_data_Salmonellosis_2countriesmissing.dta", clear


***********************************
* Preparing the data for analysis *
***********************************

// First I code a new time variable that starts at 1 and goes to 10. 
// This will facilitate the next part of analysis.
// Note that if I am interested in the intercept, this needs to be considered carefully.
gen time = year-2006

// We save the recoded dataset:
save "ECDC_surveillance_data_Salmonellosis_2countriesmissing_recoded", replace


// Step 3: Describe the missing data
*****************************
* Describe the missing data *
*****************************

// Creating a dummy variable for "being missing", which will help with the descriptive analysis
gen missing=.
recode missing (.=1) if count==.
recode missing (.=0) if count!=.

// Overall proportion of missing data
tab missing, miss

// Table of missing values over time and country
* To do this, we need to reshape the data from long to wide format
* We keep only relevant variables
keep year country missing
* Reshaping:
reshape wide missing, i(country) j(year)

// Opening original dataset again
use "ECDC_surveillance_data_Salmonellosis_2countriesmissing_recoded", clear

// Creating a dummy variable for "being complete", which will help with the descriptive analysis
gen complete=.
recode complete (.=0) if count==.
recode complete (.=1) if count!=.

// Overall proportion of missing
tab complete

// Completeness for individual years
bys year: egen num = total(complete)
bys year: egen den = total(1)
gen pc = 100 * num/den
* Output in table (number complete, total, percentage)
table year , c(sum complete count complete max pc ) format(%9.0f)

// Completeness for individual countries
capture drop num den pc
bys country: egen num = total(complete)
bys country: egen den = total(1)
gen pc = 100 * num/den
* Output in table (number complete, total, percentage)
table country , c(sum complete count complete max pc ) format(%9.0f)

// Dropping variables we no longer need
capture drop num den pc complete 


*********************
* Plotting the data *
*********************

// sorting data first, so that the plots are OK
sort year

// We can plot the countries altogether:
line count year , xtitle("Year", size(small)) ytitle("Salmonella confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(allcountries, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) ylabel(,nogrid)
** Saving the graph
graph export "images/Salmonella confirmed cases all countries.png", replace

// Or we can plot half of the countries on one graph, splitting the countries in half
// by checking how many countries are available and taking the approximate half
encode country, gen(country_num)
sum country_num
local HALF = floor(r(max)/2)

// Graphing the first half
line count year if country_num<=`HALF', xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(firsthalf, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) 

** Saving the graph
graph export "images/Salmonella confirmed cases first half of countries.png", replace

// Graphing the second half
line count year if country_num>`HALF', xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(secondhalf, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) 
** Saving the graph
graph export "images/Salmonella confirmed cases second half of countries.png", replace

// We can also plot specifically those countries with missing data:
* Making an indicator for a country having some missing data:
bys country: egen missing = sum(count==.)
scatter count year if missing>0, msize(small) mcolor("101 179 46") lcolor("101 179 46") xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(missing, replace) ylabel(, labsize(vsmall)) subtitle(,bcolor("101 179 46") color(white))  ylabel(,nogrid) 
** Saving the graph
graph export "images/Salmonella confirmed cases countries with missing values.png", replace


// Step 4: Choose the approach for handling the missing data
// Step 5: Carry out a complete case analysis
// Step 8: Estimate the trend
*********************
* Calculating trend *
*********************

// We decide to do a complete case analysis and include only countries that have no missing data
levelsof country if count==., local (COUNTRIES)
foreach COUNTRY in `COUNTRIES' {
drop if country=="`COUNTRY'"
}
 
// Random-effects meta-analysis

* Capturing the number of countries
tab country
local SUM = r(r)

* Creating matrix with the same number of rows as there are countries
* And with two columns
matrix A=J(`SUM',2,0)

* Giving rownames
gen countryadapted= subinstr(country," ","",.)
levelsof countryadapted, local(COUNTRIES)
matrix rownames A`mat'= `COUNTRIES'

* Giving column names
matrix colnames A= "Coeff" "SE" 

* Capturing coefficient and Standard Error
levelsof country, local(COUNTRIES)
local j = 1
foreach COUNTRY in `COUNTRIES' {
regress count time if country=="`COUNTRY'"
matrix A[`j',1]=_b[time]
matrix A[`j',2]=_se[time]
local j = `j' + 1
}

* checking it all looks OK
matrix list A

* Changing matrix to data
svmat2 A, names(col) rnames(study)
order study, before(Coeff)

keep study Coeff SE

// Stata 16 commands for meta-analysis
meta set Coeff SE, studylabel(study) 
meta summarize, random(dlaird) sort(_meta_id, ascending)
meta forestplot, random(dlaird) sort(_meta_id, ascending)


capture log close



