**********************************************************************
* Project: ECDC TECHNICAL REPORT: Trend analysis guidance for surveillance data
* Example 3.2 Salmo multiple imputation.do
* Date created: October 2023
* Purpose:
/*
This do-file takes the reader of the Trend analysis guidance document through
each step of the analysis for example 3.2, where we carry out a trend analysis
for with yearly count data where there is a small amount of missing data for some countries.
Please ensure your working directory is correctly set and you have a folder for log 
files and image files.
*/
*
**********************************************************************

// v1
// - Original working version
* Add here any major changes to the do-file

version 16.1


********************
* Opening log file *
********************

capture log close
log using "log/Example 3.2 Salmo multiple imputation", text replace


*********************
* Opening data file *
*********************

// Here we open an already prepared dataset, where we have coded 5 values to missing 
// We also mask the country name here, as in fact, no country has actually reported missing data
use "ECDC_surveillance_data_Salmonellosis_5missing.dta", clear


***********************************
* Preparing the data for analysis *
***********************************

// First I code a new time variable that starts at 1 and goes to 10. 
// This will facilitate the next part of analysis.
// Note that if I am interested in the intercept, this needs to be considered carefully.
gen time = year-2006

// We save the recoded dataset:
save "ECDC_surveillance_data_Salmonellosis_5missing_recoded", replace


// Step 3: Describe the missing data
*****************************
* Describe the missing data *
*****************************

// Creating a dummy variable for "being missing", which will help with the descriptive analysis
gen missing=.
recode missing (.=1) if count==.
recode missing (.=0) if count!=.

// Overall proportion of missing data
tab missing, miss

// Table of missing values over time and country
* To do this, we need to reshape the data from long to wide format
* We keep only relevant variables
keep year country missing
* Reshaping:
reshape wide missing, i(country) j(year)

// Opening original dataset again
use "ECDC_surveillance_data_Salmonellosis_5missing_recoded", clear

// Creating a dummy variable for "being complete", which will help with the descriptive analysis
gen complete=.
recode complete (.=0) if count==.
recode complete (.=1) if count!=.

// Completeness for individual years
bys year: egen num = total(complete)
bys year: egen den = total(1)
gen pc = 100 * num/den
* Output in table (number complete, total, percentage)
table year , c(sum complete count complete max pc ) format(%9.0f)

// Completeness for individual countries
capture drop num den pc
bys country: egen num = total(complete)
bys country: egen den = total(1)
gen pc = 100 * num/den
* Output in table (number complete, total, percentage)
table country , c(sum complete count complete max pc ) format(%9.0f)

// Dropping variables we no longer need
capture drop num den pc complete 


*********************
* Plotting the data *
*********************

// sorting data first, so that the plots are OK
sort year

// We can plot the countries altogether:
line count year , xtitle("Year", size(small)) ytitle("Salmonella confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(allcountries, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) ylabel(,nogrid)
** Saving the graph
graph export "images/Salmonella confirmed cases all countries.png", replace

// Or we can plot half of the countries on one graph, splitting the countries in half
// by checking how many countries are available and taking the approximate half
encode country, gen(country_num)
sum country_num
local HALF = floor(r(max)/2)

// Graphing the first half
line count year if country_num<=`HALF', xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(firsthalf, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) 

** Saving the graph
graph export "images/Salmonella confirmed cases first half of countries.png", replace

// Graphing the second half
line count year if country_num>`HALF', xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(secondhalf, replace) xscale(titlegap(5)) xlabel(, labsize(small) valuelabel) ylabel(,nogrid) lcolor("101 179 46") graphregion(margin(medium)  fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) ///
		plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) subtitle(,bcolor("101 179 46") color(white)) 
** Saving the graph
graph export "images/Salmonella confirmed cases second half of countries.png", replace

// We can also plot specifically those countries with missing data:
* Making an indicator for a country having some missing data:
bys country: egen missing = sum(count==.)
scatter count year if missing>0, msize(small) mcolor("101 179 46") lcolor("101 179 46") xtitle("Year", size(small)) ytitle("Salmonellosis confirmed cases", size(small)) by(country, graphregion(fcolor(white)) note("")) name(missing, replace) ylabel(, labsize(vsmall)) subtitle(,bcolor("101 179 46") color(white))  ylabel(,nogrid) 
** Saving the graph
graph export "images/Salmonella confirmed cases countries with missing values.png", replace


// Step 4: Choose the approach for handling the missing data
// Step 6: Carry out the multiple imputation
***********************
* Multiple imputation *
***********************

// Dropping data we don't need
drop time healthtopic population indicator unit txtvalue numvalue missing

// Setting the data for multiple imputation
mi set mlong

//  reshaping
mi reshape wide count, i(country country_num) j(year)

// Registering the data
mi register imputed country_num  count2*

// Imputing the data
* We use multiple imputation using chained equations and predictive mean matching using 5 nearest neighbours:
mi impute chained  (pmm, knn(5)) count2*  , add(5) rseed(101) dots

// Reshaping back to long
mi reshape long count, i( country) j(year)

// Saving the imputed dataset
save "ECDC_surveillance_data_Salmonellosis_5missing_recoded_MICE", replace


// Step 7: Carry out diagnostics after the imputation
***************************
* Checking imputed values *
***************************

// Check that we have 5 imputation datasets 
 tab _mi_m

// Graphing  observed and imputed values for those countries that have missing values
// We create 5 graphs per country, as we have 5 imputed datasets
* This local is just to help with naming graphs
  local i = 1
  
* We capture the names of the countries that have missing data  
  levelsof country if _mi_m!=0, local(COUNTRIES)
  
* We create 5 graphs for these countries  
 foreach COUNTRY in `COUNTRIES' {
     foreach NUM of numlist 1/5 {
  twoway (scatter count year if _mi_miss==0 & country=="`COUNTRY'", ytitle("Salmonellosis notifications `COUNTRY'", size(small)) msize(small) mcolor("101 179 46") lcolor("101 179 46")) ||   ///
   (scatter count year if _mi_miss==. & country=="`COUNTRY'" & _mi_m==`NUM', msize(small) mcolor("62 91 132") lcolor("62 91 132") ylabel(,nogrid)), graphregion(fcolor(white))  plotregion(margin(zero) fcolor(white) lcolor(white) ifcolor(white) ilcolor(white)) legend(label(1 "observed") label(2 "imputed")) saving("images/`COUNTRY'_`NUM'", replace) 
 }
* We combine the 5 graphs into one for clarity
   graph combine "images/`COUNTRY'_1.gph" "images/`COUNTRY'_2.gph" "images/`COUNTRY'_3.gph" "images/`COUNTRY'_4.gph" "images/`COUNTRY'_5.gph", col(3) name("country_`i'", replace) graphregion(color(white))
* We save the combined graphs
   graph export "images/`COUNTRY'.png", replace
* We increase the local we use for naming   
  local i = `i' + 1
 }
 

// Step 8: Estimate the trend
*********************
* Calculating trend *
*********************
 
// Random-effects meta-analysis, using imputed data
* For the meta-analysis we need to capture the trend estimate and its standard for each country. There are different ways to do this. Here we choose to use a matrix.
use  "ECDC_surveillance_data_Salmonellosis_5missing_recoded_MICE", clear

* Capturing the number of countries
tab country
local SUM = r(r)

* Creating matrix with the same number of rows as there are countries
* And with two columns
matrix A=J(`SUM',2,0)

* Giving rownames
gen countryadapted= subinstr(country," ","",.)
levelsof countryadapted, local(COUNTRIES)
matrix rownames A`mat'= `COUNTRIES'

* Giving column names
matrix colnames A= "Coeff" "SE" 

* Capturing coefficient and Standard Error
local j = 1
levelsof country, local(COUNTRIES)
foreach COUNTRY in `COUNTRIES' {
    di "`COUNTRY'"
mi estimate, dots: regress count year if country=="`COUNTRY'"
matrix TAB = r(table)
matrix A[`j',1]=TAB[1,1]
matrix A[`j',2]=TAB[2,1]
local j = `j' + 1
}

* checking it all looks OK
matrix list A

* Changing matrix to data
svmat2 A, names(col) rnames(study)
order study, before(Coeff)

keep study Coeff SE

// Stata 16 commands for meta-analysis
meta set Coeff SE, studylabel(study) 
meta summarize, random(dlaird) sort(_meta_id, ascending)
meta forestplot, random(dlaird) sort(_meta_id, ascending)


*********************************************
* Calculating trend: Ignoring missing values * 
*********************************************

// Then the random-effects meta-analysis using the complete case data (for comparison)
use "ECDC_surveillance_data_Salmonellosis_5missing_recoded", clear

* Capturing the number of countries
tab country
local SUM = r(r)

* Creating matrix with the same number of rows as there are countries
* And with two columns
matrix A=J(`SUM',2,0)

* Giving rownames
gen countryadapted= subinstr(country," ","",.)
levelsof countryadapted, local(COUNTRIES)
matrix rownames A`mat'= `COUNTRIES'

* Giving column names
matrix colnames A= "Coeff" "SE" 

* Capturing coefficient and Standard Error
levelsof country, local(COUNTRIES)
local j = 1
foreach COUNTRY in `COUNTRIES' {
regress count time if country=="`COUNTRY'"
matrix A[`j',1]=_b[time]
matrix A[`j',2]=_se[time]
local j = `j' + 1
}

* checking it all looks OK
matrix list A

* Changing matrix to data
svmat2 A, names(col) rnames(study)
order study, before(Coeff)

keep study Coeff SE

// Stata 16 commands for meta-analysis
meta set Coeff SE, studylabel(study) 
meta summarize, random(dlaird) sort(_meta_id, ascending)
meta forestplot, random(dlaird) sort(_meta_id, ascending)



// Step 5: Complete case analysis
*********************************************
* Calculating trend: complete case analysis * 
*********************************************

// Then the random-effects meta-analysis using the complete case data (for comparison)
use "ECDC_surveillance_data_Salmonellosis_5missing_recoded", clear

* Dropping countries with missing data
gen missingness=.
replace missingness=0 if count!=.
replace missingness=1 if count==.

* Dropping countries with missing values
levelsof(country) if missingness==1, local(COUNTRIES)
foreach COUNTRY in `COUNTRIES'{
	drop if country=="`COUNTRY'"
}

* Capturing the number of countries
tab country
local SUM = r(r)

* Creating matrix with the same number of rows as there are countries
* And with two columns
matrix A=J(`SUM',2,0)

* Giving rownames
gen countryadapted= subinstr(country," ","",.)
levelsof countryadapted, local(COUNTRIES)
matrix rownames A`mat'= `COUNTRIES'

* Giving column names
matrix colnames A= "Coeff" "SE" 

* Capturing coefficient and Standard Error
levelsof country, local(COUNTRIES)
local j = 1
foreach COUNTRY in `COUNTRIES' {
regress count time if country=="`COUNTRY'"
matrix A[`j',1]=_b[time]
matrix A[`j',2]=_se[time]
local j = `j' + 1
}

* checking it all looks OK
matrix list A

* Changing matrix to data
svmat2 A, names(col) rnames(study)
order study, before(Coeff)

keep study Coeff SE

// Stata 16 commands for meta-analysis
meta set Coeff SE, studylabel(study) 
meta summarize, random(dlaird) sort(_meta_id, ascending)
meta forestplot, random(dlaird) sort(_meta_id, ascending)




 

capture log close



